/*
 * Application interface library for the GUS driver
 * Copyright (c) by Jaroslav Kysela (Perex soft)
 */

#ifndef __LIBGUS_H
#define __LIBGUS_H

#include <malloc.h>		/* size_t type needed */
#include <fcntl.h>
#include <sys/gus.h>

  /*
   * all functions return value -1 if error occured
   */

/***************************************************************************
 *                                                                         *
 *              Native synthesizer support functions                       *
 *                                                                         *
 ***************************************************************************/

int gus_cards( void );
  /* 
   * return value:	number of GUS cards installed in system
   */
int gus_open( int card, size_t queue_buffer_size );
  /*
   * input values:	card number,
   *			size of command queue buffer (512-1MB)
   *			buffer is allocated dynamically
   * return value:	zero if success
   * note 1:		this function must be called as first
   * 			open file /dev/gus
   * note 2:		you can open more cards with one process
   */
int gus_close( int card );
  /*
   * close file (gus synthesizer) previously opened with gusOpen function
   * return value:	zero if success
   */
int gus_open_reg( int card, int samples );
  /*
   * prepare for open - allow work with samples (gus_sample_reg_XXXX fcns)
   * gus_open call after this needed!!!
   */
int gus_close_reg( int card );
  /*
   * close preparing...
   */
int gus_select( int card );
  /*
   * select specified card
   * return value:	zero if success
   */
int gus_get_handle( void );
  /*
   * return value:	file handle (descriptor) for /dev/gus
   */

int gus_info( struct GUS_STRU_INFO *info );
  /*
   * return value:	filled info variable with actual values
   * 			(look at gus.h header file for more informations)
   */
int gus_info_port( void );
  /*
   * return value:	port number (for example 0x220)
   */
int gus_info_irq( void );
  /*
   * return value:	irq number (for example 11)
   */
int gus_info_dma1( void );
  /*
   * return value:	dma1 number (for example 5)
   */
int gus_info_dma2( void );
  /*
   * return value:	dma2 number (for example 6)
   * note:		dma1 and dma2 could be same in case of only one dma channel used
   */
int gus_info_version( void );
  /*
   * return values:	0x24	- gus revision 2.4
   *			0x35	- gus revision 3.7 with flipped mixer channels
   *			0x37	- gus revision 3.7
   *			0xa0	- gus max revision 10?
   */
int gus_info_noise_channel( void );
  /*
   * return value:	0	- noise channel disabled
   *			1	- noise channel enabled
   * note:		if you use frequency type GF1 (GUS_FREQ_TYPE_GF1)
   *			you have to count in noise channel
   * warning:		reset function must be called before
   */

int gus_setup_volume_type( int type );
  /*
   * input value:	look at gus.h header file for GUS_VOL_TYPE_XXXX constants
   * return value:	zero if success
   */
int gus_setup_frequency_type( int type );
  /*
   * input value:	look at gus.h header file for GUS_FREQ_TYPE_XXXX constants
   * return value:	zero if success
   */
int gus_setup_set_auto_ramp( int speed );
  /*
   * input value:	auto ramp speed - 64 = off, 0 = max speed
   * return value:	zero if success
   */
int gus_setup_get_auto_ramp( void );
  /*
   * return value:	auto ramp speed
   */
int gus_setup_set_smooth_pan( int speed );
  /*
   * input value:	smooth pan speed - 0   = off (max speed),
   *					   255 = min speed
   * return value:	zero if success
   */
int gus_setup_get_smooth_pan( void );
  /*
   * return value:	smooth pan speed
   */
int gus_setup_noise_channel( struct GUS_STRU_NOISE * );
  /*
   * input value:	pointer to GUS_STRU_NOISE structure
   *			look at gus.h header file for more details
   * return value:	zero if success
   */

int gus_memory_size( void );
  /*
   * return value:	gus memory size in bytes
   */
int gus_memory_free( void );
  /*
   * return value:	unused gus memory in bytes
   * warning:		reset function must be called before
   */
int gus_memory_max_bank_free( void );
  /*
   * return value:	current maximum free memory in one bank
   * warning:		reset function must be called before
   */

int gus_reset( short voices, short samples );
  /*
   * input values:	active voices (without noise channel),
   *			maximum count of samples in DRAM
   * return value:	number of active voices if reset was successful (GF1 chip active)
   *			Warning! If noise channel is enabled -
   *			this value contains input + 1 voices.
   */
int gus_reset_engine_only( void );
  /*
   * return value:	same as gus_reset function
   * note:		this command doesn't change number of active
   *			voices and doesn't do hardware reset
   */
   
int gus_timer_start( void );
  /*
   * return value:	zero if successful
   */
int gus_timer_stop( void );
  /*
   * return value:	zero if timer was stoped
   */
int gus_timer_global_insert( void );
int gus_timer_global_delete( void );

int gus_sample_download( struct GUS_STRU_DOWNLOAD *download );
  /*
   * input value:	look at gus.h for more details about GUS_STRU_DOWNLOAD
   *			structure
   * return value:	zero if sample was successfully downloaded
   */
int gus_sample_download_reset( void );
  /*
   * return value:	zero if samples was successfully removed from
   *			GF1 memory manager
   */
int gus_sample_download_test( struct GUS_STRU_DOWNLOAD *download );
  /*
   * input value:	look at gus.h for more details about GUS_STRU_DOWNLOAD
   *			structure
   * return value:	zero if sample !can be! downloaded
   * note:		sample is allocated in GF1 memory manager
   */
int gus_sample_download_test1( struct GUS_STRU_DOWNLOAD *download );
  /*
   * input value:	look at gus.h for more details about GUS_STRU_DOWNLOAD
   *			structure
   * return value:	zero if sample !can be! downloaded
   * note:		sample isn't allocated in GF1 memory manager
   */

#define GUS_WAVE_UNSIGNED	GUS_WAVE_INVERT	/* alias only */
#define GUS_WAVE_NO_LOOP_ADAPT	0x00010000 /* don't adapt sample loop for GF1 chip */
#define GUS_WAVE_DELTA		0x00020000 /* sample is stored in delta format */
#define GUS_WAVE_NO_EXPAND	0x00040000 /* don't allow expansion of this sample */

int gus_sample_reg( unsigned short number, unsigned int type,
                    int size, int lstart, int lend );
  /*
   * input value:	sample number
   *			sample type (see GUS_WAVE_XXXX constants to gus.h)
   *			sample size
   *			sample loop start
   *			sample loop end
   * note: 		gus_sample_reg_XXXX operates only with samples 
   *			registered with this function
   */
int gus_sample_reg_size( void );
  /*
   * output value:	total size of registered samples in gus memory
   */
int gus_sample_reg_download( 
	int (*read_fcn)( unsigned short number, unsigned char *ptr, int size ),
	void (*error_fcn)( unsigned short number, int error ) );
  /*
   * input value:	pointer to function which reads sample to *ptr address;
   *			function returns zero if successful
   */

#define GUS_SAMPLE_EXPAND_NONE	0
#define GUS_SAMPLE_EXPAND_S8	1	/* spline 8-bit */
#define GUS_SAMPLE_EXPAND_S16	2	/* spline 16-bit */

int gus_sample_reg_expand( int type );
  /*
   * input value:	expand type - see GUS_SAMPLE_EXPAND_XXXX
   * note:		this function expands or shrinks all registered
   *			samples in gus memory size
   */
int gus_sample_reg_prepare_reset( void );
  /*
   * result:		reset of sample prepare system
   */
int gus_sample_reg_prepare_download(
		void (*error_fcn)( unsigned short number, int error ) );
  /*
   * result:		download prepared samples in GUS's memory
   */

int gus_queue_size( void );
  /*
   * return value:	queue size in items (ticks) - default value is 64
   */
int gus_queue_free( void );
  /*
   * return value:	free items (ticks) in queue
   */
int gus_queue_threshold( int threshold );
  /*
   * input value:	threshold in queue items (ticks)
   *			range 1 - gus_queue_size()
   *			standard value = gus_queue_size()
   *			good value = gus_queue_size - ( gus_queue_size() / 3 )
   * return value:	zero if success
   */
int gus_queue_flush( void );
  /*
   * return value:	zero if command queue was successfully flushed
   */
int gus_queue_abort( void );
  /*
   * return value:	zero if command queue was successfully aborted and
   *			GF1 chip was stoped
   * note:		reset must be called after this command for
   *			more sound!
   */
int gus_queue_abort_to_stop( void );
  /*
   * return value:	zero if command queue was successfully aborted
   *			to queue command STOP; all voices was stoped
   */
int gus_queue_stop_playback( void );
  /*
   *			zero if command was successful
   */
int gus_queue_continue_playback( void );
  /*
   *			zero if command was successful
   */
int gus_queue_set_tick( unsigned int *tick );
  /*
   * input value:	tick count (only informational value for application)
   * return value:	zero if successful
   */
int gus_queue_get_tick( unsigned int *tick );
  /*
   * return value:	current tick count ( in *tick )
   */
int gus_queue_set_echo_items( int items );
  /*
   * input value:	echo buffer size in items (if 0 - erase echo buffer)
   */
int gus_queue_get_echo( unsigned char *dest, int size );
  /*
   * return value:	size of echo command or -1 if error
   */

/*
 *  Queue commands
 *
 *  This commands are interpreted by block. Blocks are separated by command
 *  gus_do_wait or gus_do_stop (for abort to stop command). Queue command
 *  isn't sent to GUS driver in same time user calls it. Queue commands
 *  are stored into internal buffer dynamically allocated by gus library
 *  (look at gus_open function). Internal buffer is flushed (forwarded to
 *  GUS driver) when is full or in case user forces buffer flush with
 *  gus_do_flush function.
 */

int gus_do_flush( void );
  /*
   * return value:	zero if command queue was successfully flushed
   */

void gus_do_voice_select( unsigned char voice );
  /*
   * select active voice (0-31)
   */
void gus_do_sample_select( unsigned short sample );
  /*
   * select active sample (0-127)
   */
void gus_do_voice_control( unsigned char cntrl );
  /*
   * set voice control register:
   *	bit 0. - 0 = voice is running, 1 = voice is stopped
   *	bit 1. - 0 = nothing, 1 = stop voice
   *	bit 2. - 0 = 8 bit wave, 1 = 16 bit wave
   *	bit 3. - 0 = nothing, 1 = enable looping
   *	bit 4. - 0 = nothing, 1 = bidirectional looping
   *	bit 5. - 0 = nothing, 1 = enable wavetable IRQ
   *	bit 6. - 0 = increasing addresses, 1 = decreasing addresses
   *	bit 7. - 0 = wavetable IRQ pending
   * note: don't set bits 5 & 7 - these things must be controled by driver
   */
void gus_do_frequency( unsigned int freq );
  /*
   * set voice frequency:
   *	frequency type = GUS_FREQ_TYPE_GF1 - native gus frequency
   *    frequency type = GUS_FREQ_TYPE_HZ2 - frequency in Hz * 2
   *    frequency type = GUS_FREQ_TYPE_HZ  - frequency in Hz (default)
   */
void gus_do_loop_start( unsigned int start );
  /*
   * set voice loop start position in sample
   * value is relative from the begining of sample
   */
void gus_do_loop_end( unsigned int end );
  /*
   * set voice loop end position in sample
   * value is relative from the begining of sample
   */
void gus_do_ramp_rate( unsigned char rate );
  /*
   * set voice volume ramping rate
   * (look at UltraSound Lowlevel Toolkit documentation for more details)
   *    bits 7-6: rate
   *	bits 5-0: volume increment
   */
void gus_do_ramp_start( unsigned char start );
  /*
   * set voice ramp start value
   *    bits 7-4: exponent
   *    bits 3-0: mantisa
   */
void gus_do_ramp_end( unsigned char end );
  /*
   * set voice ramp end value
   *    bits 7-4: exponent
   *    bits 3-0: mantisa
   */
void gus_do_volume( unsigned short vol );
  /*
   * set voice volume level:
   *	volume type = GUS_VOL_TYPE_GF1 - native gus volume level
   *    volume type = GUS_VOL_TYPE_LINEAR - linear (0-128)
   */
void gus_do_position( unsigned int pos );
  /*
   * set current voice position
   * value is relative from the begining of sample
   */
void gus_do_pan( unsigned char pan );
  /*
   * set voice pan
   */
void gus_do_ramp_control( unsigned char cntrl );
  /*
   * set ramp control register
   *   bit 0. - 0 = volume ramping in progress, 1 = volume ramping is inactive
   *   bit 1. - 0 = nothing, 1 = stop ramp
   *   bit 2. - 0 = nothing, 1 = roll over condition
   *   bit 3. - 0 = nothing, 1 = ramp loop enable
   *   bit 4. - 0 = nothing, 1 = bidirectional ramp loop enable
   *   bit 5. - 0 = nothing, 1 = volume ramp IRQ enable
   *   bit 6. - 0 = increasing direction, 1 = decreasing direction
   *   bit 7. - 0 = nothing, 1 = volume ramp IRQ pending
   * note: don't set bits 2,5,7 - these things must be controled by driver
   */
void gus_do_freq_and_vol( unsigned int freq, unsigned short vol );
  /*
   * combined command - frequency and volume
   */
void gus_do_loop_all( unsigned char cntrl,
                      unsigned int start,
                      unsigned int end );
  /*
   * combined command - voice control, loop start, loop end
   */
void gus_do_ramp_all( unsigned char cntrl, unsigned char rate,
                      unsigned char start, unsigned char end );
  /*
   * combined command - volume ramp control, ramp rate, ramp start, ramp end
   */
void gus_do_voice_stop( void );
  /*
   * voice stop command
   */

   
  /*
   *  gus_do_vs_XXXX commands are equivalents to gus_do_XXXX commands, except
   *  that first argument is voice (channel);
   *  combination of gus_do_voice_select + gus_do_XXXX
   */
   
void gus_do_vs_voice_control( unsigned char voice, unsigned char cntrl );
void gus_do_vs_frequency( unsigned char voice, unsigned int freq );
void gus_do_vs_loop_start( unsigned char voice, unsigned int start );
void gus_do_vs_loop_end( unsigned char voice, unsigned int end );
void gus_do_vs_ramp_rate( unsigned char voice, unsigned char rate );
void gus_do_vs_ramp_start( unsigned char voice, unsigned char start );
void gus_do_vs_ramp_end( unsigned char voice, unsigned char end );
void gus_do_vs_volume( unsigned char voice, unsigned short vol );
void gus_do_vs_position( unsigned char voice, unsigned int pos );
void gus_do_vs_pan( unsigned char voice, unsigned char pan );
void gus_do_vs_ramp_control( unsigned char voice, unsigned char cntrl );
void gus_do_vs_freq_and_vol( unsigned char voice,
                             unsigned int freq,
                             unsigned short vol );
void gus_do_vs_loop_all( unsigned char voice,
                         unsigned char cntrl,
                         unsigned int start,
                         unsigned int end );
void gus_do_vs_ramp_all( unsigned char voice,
                         unsigned char cntrl, unsigned char rate,
                         unsigned char start, unsigned char end );
void gus_do_vs_voice_stop( unsigned char voice );


void gus_do_sample_start( unsigned char voice, unsigned short sample,
                          unsigned int freq, unsigned short volume );
  /*
   * start playback of sample
   */
void gus_do_sample_start1( void );
  /*
   * start playback of sample, but all things must be initialized manually
   * this comes in account after gus_do_stop command
   */
void gus_do_wait( int ms );
  /*
   * wait x microseconds - this command is block separator
   * all commands between blocks are interpreted in the begining of one tick
   */
void gus_do_stop( void );
  /*
   * this command is remark for gus_queue_abort_to_stop
   */
void gus_do_echo( unsigned char *src, int size );
  /*
   * echo data at *src in specified time to file (stream) input
   */

/***************************************************************************
 *                                                                         *
 *                    GUS mixer support functions                          *
 *                                                                         *
 ***************************************************************************/

#define GUS_MIXER_NONE		0	/* GUS rev. <3.7 */
#define GUS_MIXER_ICS		1	/* GUS rev. 3.7 & above */
#define GUS_MIXER_CODEC		2	/* GUS MAX or 16-bit Daughter Board */
#define GUS_MIXER_ENH_CODEC	3	/* enhanced codec - GUS PnP */

int gus_mixer_cards( void );
  /*
   * return value:	cards installed in the system
   */
int gus_mixer_open( int card );
  /*
   * input value:	selected card
   * return value:	selected card if successful
   */
int gus_mixer_close( int card );
  /*
   * input value:	card to close
   * return value:	zero if successful
   */
int gus_mixer_select( int card );
  /*
   * input value:	card to select
   * return value:	zero if successful
   */

int gus_mixer_version( void );
  /*
   * return value:	see to GUS_MIXER_XXXX constants
   */
int gus_mixer_read_devmask( void );
  /*
   * return value:	available devices mask
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_read_stereodevs( void );
  /*
   * return value:	available stereo devices mask
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_read_recmask( void );
  /*
   * return value:	available record devices mask
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_read_recsrc( void );
  /*
   * return value:	current record source
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_write_recsrc( int value );
  /*
   * input value:	new record source
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_read_devs_lmute( void );
  /*
   * return value:	bitmask with muted devices (left channel)
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_read_devs_rmute( void );
  /*
   * return value:	bitmask with muted devices (right channel)
   *			(look at SOUND_MASK_XXXX - soundcard.h)
   */
int gus_mixer_write_devs_lmute( int dev, int mute );
  /*
   * input value:	device and mute flag ( 0 - unmute, != 0 - mute)
   *			left channel
   *			(look at SOUND_MIXER_XXXX - soundcard.h)
   * return value:	zero if successful
   */
int gus_mixer_write_devs_rmute( int dev, int mute );
  /*
   * input value:	device and mute flag ( 0 - unmute, != 0 - mute)
   *			right channel
   *			(look at SOUND_MIXER_XXXX - soundcard.h)
   * return value:	zero if successful
   */

  /*
   * functions bellow use following form of the input/output values:
   *	bits 0-7 (low byte): left channel volume level (0-100)
   *	bits 8-15 (high byte): right channel volume level (0-100)
   */

int gus_mixer_read_mic( void );		/* all (NONE & CODEC - mono) */
int gus_mixer_read_cd( void );		/* ICS, CODEC, ENH_CODEC */
int gus_mixer_read_line( void );	/* all (NONE - mono) */
int gus_mixer_read_synth( void );	/* all (NONE - mono) */
int gus_mixer_read_pcm( void );		/* all */
int gus_mixer_read_reclev( void );	/* CODEC, ENH_CODEC - mono */
int gus_mixer_read_volume( void );	/* ICS, ENH_CODEC */
int gus_mixer_read_imix( void );	/* ICS, CODEC - both mono */

int gus_mixer_write_mic( int value );
int gus_mixer_write_cd( int value );
int gus_mixer_write_line( int value );
int gus_mixer_write_synth( int value );
int gus_mixer_write_pcm( int value );
int gus_mixer_write_reclev( int value );
int gus_mixer_write_volume( int value );
int gus_mixer_write_imix( int value );

/***************************************************************************
 *                                                                         *
 *                      Conversion routines                                *
 *                                                                         *
 ***************************************************************************/

void gus_convert_delta( unsigned int type, unsigned char *dest,
			unsigned char *src, size_t size );
  /*
   * note: dest and src pointers can be equal
   */
void gus_convert_8bit_to_16bit( unsigned short *dest, unsigned char *src,
				size_t size );
  /*
   * note: dest and src pointers can be equal
   */
void gus_convert_spline_oversampling_8bit(
			signed char *dest, signed char *src,
			size_t size, float factor, int unsigned_ );
  /*
   * note: output array is always signed
   */
void gus_convert_spline_oversampling_16bit(
			signed short *dest, signed short *src,
			size_t size, float factor, int unsigned_ );
  /*
   * note: output array is always signed
   */
void gus_convert_spline_oversampling_8bit_to_16bit(
			signed short *dest, signed char *src,
			size_t size, float factor, int unsigned_ );
  /*
   * note: output array is always signed
   */

/***************************************************************************
 *                                                                         *
 *                             PCM routines                                *
 *                                                                         *
 ***************************************************************************/

int gus_pcm_cards( void );
  /*
   * return value:	cards installed in the system
   */

#define GUS_PCM_DIRECTION_PLAY		O_RDONLY
#define GUS_PCM_DIRECTION_RECORD	O_WRONLY
#define GUS_PCM_DIRECTION_DUPLEX	O_RDWR

int gus_pcm_open( int card, int direction );
int gus_pcm_close( int card );
int gus_pcm_get_handle( int card );

int gus_pcm_select_card( int card );

#define GUS_PCM_MODE_S8			AFMT_S8
#define GUS_PCM_MODE_U8			AFMT_U8
#define GUS_PCM_MODE_S16_LE		AFMT_S16_LE	/* little endian */
#define GUS_PCM_MODE_U16_LE		AFMT_U16_LE	/* little endian */
#define GUS_PCM_MODE_S16_BE		AFMT_S16_BE	/* big endian */
#define GUS_PCM_MODE_U16_BE		AFMT_U16_BE	/* big endian */
#define GUS_PCM_MODE_MU_LAW		AFMT_MU_LAW
#define GUS_PCM_MODE_A_LAW		AFMT_A_LAW
#define GUS_PCM_MODE_IMA_ADPCM		AFMT_IMA_ADPCM
#define GUS_PCM_MODE_STEREO		0x80000000	/* only this is switch */

int gus_pcm_set_mode( unsigned int mode );
unsigned int gus_pcm_get_mode( void );
unsigned int gus_pcm_supported_modes( void );

int gus_pcm_set_rate( int rate );
int gus_pcm_get_rate( void );

int gus_pcm_set_dma_size( int size );
int gus_pcm_get_dma_size( void );
int gus_pcm_get_max_dma_size( void );

int gus_pcm_sync( void );

int gus_pcm_read( char *buffer, int count );
int gus_pcm_write( char *buffer, int count );

/***************************************************************************
 *                                                                         *
 *                             MIDI routines                               *
 *                                                                         *
 ***************************************************************************/

#define GUS_MIDI_CHANNELS	16

#define GUS_MCTL_BANK_SELECT		0x00
#define GUS_MCTL_MODWHEEL		0x01
#define GUS_MCTL_BREATH			0x02
#define GUS_MCTL_FOOT			0x04
#define GUS_MCTL_PORTNAMENTO_TIME	0x05
#define GUS_MCTL_DATA_ENTRY		0x06
#define GUS_MCTL_MAIN_VOLUME		0x07
#define GUS_MCTL_BALANCE		0x08
#define GUS_MCTL_PAN			0x0a
#define GUS_MCTL_EXPRESSION		0x0b
#define GUS_MCTL_GENERAL_PURPOSE1	0x10
#define GUS_MCTL_GENERAL_PURPOSE2	0x11
#define GUS_MCTL_GENERAL_PURPOSE3	0x12
#define GUS_MCTL_GENERAL_PURPOSE4	0x14
#define GUS_MCTL_SUSTAIN		0x40
#define GUS_MCTL_PORNAMENTO		0x41
#define GUS_MCTL_SOSTENUTO		0x42
#define GUS_MCTL_SOFT_PEDAL		0x43
#define GUS_MCTL_HOLD2			0x45
#define GUS_MCTL_GENERAL_PURPOSE5	0x50
#define GUS_MCTL_GENERAL_PURPOSE6	0x51
#define GUS_MCTL_GENERAL_PURPOSE7	0x52
#define GUS_MCTL_GENERAL_PURPOSE8	0x53
#define GUS_MCTL_EXT_EFF_DEPTH		0x54
#define GUS_MCTL_TREMOLO_DEPTH		0x5b
#define GUS_MCTL_CHORUS_DEPTH		0x5c
#define GUS_MCTL_DETUNE_DEPTH		0x5e
#define GUS_MCTL_PHASER_DEPTH		0x5f
#define GUS_MCTL_DATA_INCREMENT		0x60
#define GUS_MCTL_DATA_DECREMENT		0x61
#define GUS_MCTL_NONREG_PARM_NUM_LSB	0x62
#define GUS_MCTL_NONREG_PARM_NUM_MSG	0x63
#define GUS_MCTL_REGIST_PARM_NUM_LSB	0x64
#define GUS_MCTL_REGIST_PARM_NUM_MSB	0x65

#define GUS_MIDI_F_EXTERNAL	0x00000001	/* output is external synthesizer (connected to midi port) */
#define GUS_MIDI_F_PNP		0x00000002	/* GUS PnP */
#define GUS_MIDI_F_OUTPUT	0x00010000	/* output enabled */
#define GUS_MIDI_F_INPUT	0x00020000	/* input enabled (maybe only external) */
#define GUS_MIDI_F_8BIT		0x00040000	/* use 8-bit patches */

#define GUS_MIDI_E_SYS_PREFIX	0xf0

typedef struct __gus_midi_card {
  __u32 flags;
  __s32 card;			/* card # */
  __u16 channels; 		/* midi channels processed by this card (bitmap) */
  __u8  drum_channel;		/* drum channel */
  size_t buffer_size;		/* queue buffer size */
  char map_filename[ 257 ];	/* maybe null */
  char pgm_filename[ 257 ];	/* maybe null */
  struct __gus_midi_card *next;	/* next card */
} gus_midi_card_t;

char gus_midi_error[];

int gus_midi_cards( void );
int gus_midi_external_ios( void );	/* ios = input/output systems */

int gus_midi_open( gus_midi_card_t *cards );
int gus_midi_open_intelligent( char *cfg_file );
int gus_midi_close( void );

int gus_midi_device_count( void );
int gus_midi_device_info( int device );

int gus_midi_reset( void );

void gus_midi_note_on( int chn, int note, int vel );
void gus_midi_note_off( int chn, int note, int vel );
void gus_midi_key_pressure( int chn, int note, int vel );
void gus_midi_chn_pressure( int chn, int note, int vel );
void gus_midi_bender( int chn, int pitch );	/* range: 0-32768 */
void gus_midi_pgm_change( int chn, int pgm );
void gus_midi_pgm_preload( int chn, int pgm );
void gus_midi_control( int chn, int p1, int p2 );

void gus_midi_main_volume( int volume );	/* 0-128 */

void gus_midi_sysex( int device, int cmd, unsigned char *data, int len );

void gus_midi_timer_start( void );
void gus_midi_timer_stop( void );
void gus_midi_timer_continue( void );
void gus_midi_wait( int ms );
void gus_midi_write( void );
void gus_midi_flush( void );

/*
 *  functions for patch interface...
 */

#define GUS_MIDI_MAX_WAVE_NUMBER	256
#define GUS_MIDI_MAX_INSTR_NUMBER	256	/* don't change this value!!! */	

#if GUS_MIDI_MAX_WAVE_NUMBER > GUS_MIDI_MAX_INSTR_NUMBER
#error "!!!!gus_midi_patches_t.flags exceed!!!!"
#endif

#define GUS_MIDI_PATCH_M_INITIALIZED	0x00000001
#define GUS_MIDI_PATCH_M_PRELOAD	0x00000002
#define GUS_MIDI_PATCH_M_8BIT		0x00000004

#define GUS_MIDI_WAVE_F_ENVELOPES	0x0001

#define GUS_MIDI_INSTR_F_LOADED		0x01
#define GUS_MIDI_INSTR_F_NOMEMORY	0x02

typedef struct {
  __u16 flags;
  __u32 sample_rate;
  __u32 low_frequency;
  __u32 high_frequency;
  __u32 root_frequency;
  __s16 tune;
  __u8  panning;
  __s16 next;			/* next sample # */
} gus_midi_wave_t;

typedef struct {
  __u16 flags;
  __u16 master_volume;
  __s16 wave;			/* wave index */
} gus_midi_instr_t;

typedef struct {
  __u32 freq;
} gus_midi_pvoice_t;

typedef struct {
  __u32 mode;
  __s32 card;
  __s32 waves_used;
  gus_midi_wave_t waves[ GUS_MIDI_MAX_WAVE_NUMBER ];
  gus_midi_instr_t instrs[ GUS_MIDI_MAX_INSTR_NUMBER ];
  gus_midi_pvoice_t voices[ 32 ];
  char *pgm_filename;		/* filename to translation of instruments to patches */
} gus_midi_patches_t;

void gus_patch_init( gus_midi_patches_t *, int card, int mode );
int gus_patch_download_init( gus_midi_patches_t * );
int gus_patch_download_done( gus_midi_patches_t * );
int gus_patch_download( gus_midi_patches_t *, int pgm );

int gus_patch_voice_on( gus_midi_patches_t *, int voice, int instr, int note, int vel );
int gus_patch_voice_off( gus_midi_patches_t *, int voice );
int gus_patch_bender( gus_midi_patches_t *, int voice, int value, int range );
int gus_patch_volume( gus_midi_patches_t *, int voice, int volume );
int gus_patch_pan( gus_midi_patches_t *, int voice, int pan );

int gus_patch_envelope_continue( gus_midi_patches_t *, int instr, int note );

/*
 *  warning!! those functions are not yet implemented... maybe in near future..
 */

int gus_external_open( int card, size_t buffer_size );
int gus_external_close( int card );
int gus_external_select( int card );

int gus_external_note_on( int chn, int note, int vel );
int gus_external_note_off( int chn, int note, int vel );
int gus_external_control( int chn, int controller, int value );
int gus_external_sysex( int cmd, unsigned char *data, int len );

int gus_external_wait( int ms );
int gus_external_flush( void );
 
/*
 *
 */
 
#endif /* __LIBGUS_H */
